/**********************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: Profiler.java,v 1.10 2005/04/15 15:57:16 rdanek Exp $
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.hyades.collection.profiler;

import java.util.ArrayList;
import java.util.List;

/**
 * This is a singleton class that is used by the application to manage the Java Profiling
 * Agent.  This class is a singleton and cannot be instantiated by the user.  In order
 * to access the singleton the application must use Profiler.getProfiler()
 * @author: Richard K. Duggan
 */
public final class Profiler {

	
	/* Boolean flag that is set only when the profiler is actively tracing */
	private static boolean _isTracing = true;
	
	/* Boolean flag to determine if the profiler was successful linked. */
	private static boolean _isActive=true;
	
	/* Boolean flag to determine if we are currently profiling */
	private boolean _isProfiling=false;
	
	/* The mode we are running in */
	private int _mode=PROFILER_EXECUTION_AND_HEAP; 
	
	/* The count of current active gc disable calls */
	private int _gcCount=0;
	
	private List _fileList;
	
	/* The singleton instance of Profiler */
	private static Profiler _profiler;
	
	/* The global exception indicating why the profiler is unavailable */
	private static ProfilerNotAvailableException _errorCondition;
	
	
	
	
	/* The various modes that are supported */
	public static final int PROFILER_EXECUTION_ONLY 			= 1;
	public static final int PROFILER_HEAP_ONLY 					= 2;
	public static final int PROFILER_EXECUTION_AND_HEAP			= 3;
	public static final int PROFILER_OPTIMIZED_HEAP_ONLY		= 4;
	
	
	/* Various return values from emitXMLFragment() */ 
	public static final int EMIT_XML_SUCCESS					= 0; /* indicates success */ 
	public static final int EMIT_XML_SUSPENDED_IO				= 1; /* failure because of monitoring being off */  
	public static final int EMIT_XML_FAIL						= 2; /* failure for other reasons */  
	
	
	/* Link to the profiling agent.*/
	static {
	
		try {
			System.loadLibrary("piAgent");
			_profiler=new Profiler();
		}
		catch(UnsatisfiedLinkError f) {
			_isActive=false;
			_profiler=null;
			_errorCondition=new ProfilerNotAvailableException("Cannot locate Java Profiling Agent.  The Agent Controller must be"
															  +" installed and the piAgent library must be provided as"
			                                                  +" a parameter to the JVM and it must be running in \"application\" mode."
			                                                  +" ie. -XrunpiAgent:server=application must be provided as a parameter to"
			                                                  +" the JVM: "+f);
		}
		catch(ProfilerNotAvailableException g) {
			_isActive=false;
			_profiler=null;	
		}
		
	}
	
	
	/**
	 * Don't allow users to make an instance of this class.
	 */
	private Profiler()  throws ProfilerNotAvailableException  { 	
		int result=initialize0();
		
		/* Did we startup OK.  If not set the error in the global exception object */
		if(result<0) {
			_errorCondition=new ProfilerNotAvailableException("The Java Profiling Agent is currently not running in \"application\""
				                                         +" mode. ie. -XrunpiAgent:server=application must be provided as a "
				                                         +" parameter to the JVM");
			throw _errorCondition;
		}
	}
	
	/**
	 * Native delegate to startup the profiler.  Checks to see if profiling is possible.
	 */
	 private native int initialize0();
	
	/**
	 * Getter to retieve the singleton Profiler.
	 */
	public static Profiler getProfiler() throws ProfilerNotAvailableException {
		if(!_isActive) {
			throw _errorCondition;
		}
		return _profiler;  	
	}
	
	
	/**
	 * Once a handle to the profiler is established the mode in which it is running can be set.  The default is PROFILER_EXECUTION_AND_HEAP.
	 * @param mode
	 * @throws ProfilerNotAvailableException
	 */
	public void setMode(int mode) throws ProfilerNotAvailableException {
		if(mode < 1 || mode > 4) {
			throw new ProfilerNotAvailableException("Unknown mode");
		}
		else if(mode==PROFILER_EXECUTION_ONLY) {
			
		}
		else if (mode==PROFILER_HEAP_ONLY) {
			
		}
		else if(mode==PROFILER_EXECUTION_AND_HEAP) {
			
		}
		else if(mode==PROFILER_OPTIMIZED_HEAP_ONLY) {
			if(_mode!=PROFILER_OPTIMIZED_HEAP_ONLY) {
				throw new ProfilerNotAvailableException("Unsupported mode change request");
			}
		}
		setMode0(mode);
	}
	
	/**
	 * Native delegate of optimizedHeapDump
	 */
	private native void setMode0(int mode);
	
	/**
	 * Start a profiling session.  If a profiling session is currently executing the call is ignored.
	 *
	 * @param currentThreadOnly - the profiling information is by default for the entire process.  If
	 *                             currentThreadOnly is true profiling information will be restricted
	 *                             to the thread that is currently executing the startProfiling method.
	 */
	public void startProfiling(boolean currentThreadOnly) {
		startProfiling(currentThreadOnly, 0);
	}
	
	/**
	 * Start a profiling session.  If a profiling session is currently executing the call is ignored.
	 *
	 * @param currentThreadOnly - the profiling information is by default for the entire process.  If
	 *                             currentThreadOnly is true profiling information will be restricted
	 *                             to the thread that is currently executing the startProfiling method.
	 *
	 * @param boundaryDepth - the profiling information does not extend beyond the number of frames
	 *                         indicated by boundaryDepth.  If the boundary depth is zero or negative
	 *                         then the number of frames is unlimited.
	 */
	public void startProfiling(boolean currentThreadOnly, int boundaryDepth) {
		synchronized(this) {
			if(_mode==PROFILER_EXECUTION_ONLY || _mode==PROFILER_EXECUTION_AND_HEAP) {
				if(!_isProfiling) {
					if(startProfiling0(currentThreadOnly, boundaryDepth)==0) {
						_isProfiling=true;
					}
				}
			}
		}
	}
	
	/**
	 * Native delegate of startProfiling(boolean currentThreadOnly, int bondaryDepth).
	 * @returns  0 - the profile session was created successfuly.
	 *
	 */
	private native int startProfiling0(boolean currentThreadOnly, int boundaryDepth);
	
	
	
	/**
	 * Stop profiling the information requested in a previous call to startProfiling.  If there is no
	 * current profiling session the call is ignored.  This call need not be on the same thread as the
	 * startProfiling call irregardless of whether singlethreaded profiling was requested.
	 *
	 */
	public void stopProfiling() {
		synchronized(this) {
			if(_isProfiling) {
				stopProfiling0();
				_isProfiling=false;
			}
		}
	}
	
	/**
	 * Native delgate to stopProfiling.
	 */
	private native void stopProfiling0();

	/**
	  * Mark the heap.  Marking the heap causes all the objects currently in the heap to be marked
	  * so that a subsequent call to analyzeHeap will only apply to the newly created objects between
	  * the two calls.
	  */
	public void markHeap(){
		if(_mode==PROFILER_HEAP_ONLY || _mode==PROFILER_EXECUTION_AND_HEAP ) {
			if(_isActive) {
				markHeap0();
			}
		}
	}
	
	/**
	 * Native delegate for markHeap()
	 */
	private native void markHeap0();
	
	/**
	 * Analyze the heap to determine what new objects have been created in the heap since the last
	 * time the heap was marked or the last time it was analyzed.
	 * @param name - a label that will be applied to this analysis session so that it can be identified
	 *                amongst several analysis sessions.  THIS IS CURRENTLY NOT SUPPORTED.
	 */
	public void analyzeHeap(String name){
		if(_mode==PROFILER_HEAP_ONLY || _mode==PROFILER_EXECUTION_AND_HEAP ) {
			if(_isProfiling) {
				analyzeHeap0(name);
			}
		}
	}
	
	/**
	 * Native delegate of analyzeHeap
	 */
	private  native void analyzeHeap0(String name);
	
	
	/**
	 * Take a optimized heap dump of the application.  Each snapshot will be placed in a series of files.  These files must
	 * be retrieved by the tooling for analysis.  This retrieval is beyond the scope of this profiler.
	 * @return - the list of files that contain optimized heap dump information.  
	 */
	public List optimizedHeapInfoSnapshot() {
		if(_mode==PROFILER_OPTIMIZED_HEAP_ONLY) {
			if(_fileList==null) {
				_fileList=new ArrayList();
			}
			_fileList.add(optimizedHeapDump0());
			
		}
		return null;
	}
	
	/**
	 * Native delegate of optimizedHeapDump
	 */
	private native String optimizedHeapDump0();
	
	public List stopOptimizedHeapInfoSnapshot() {
		if(_mode==PROFILER_OPTIMIZED_HEAP_ONLY) {
			_fileList.add(stopOptimizedHeapInfoSnapshot0());
			List result=_fileList;
			_fileList=null;
			return result;
		}
		
		return null;
	}
	
	/**
	 * Native delegate of stopOptimizedHeapInfoSnapshot
	 */
	private native String stopOptimizedHeapInfoSnapshot0();
	
	
	/**
	 *  Disable garbage collection.  A subsequent call to enableGC() will be required to reenable GC.
	 */
	public void disableGC() {
		if(_isActive) {
			_gcCount++;
			disableGC0();
		}
	}
	
	private native void disableGC0();
	
	
	/**
	 *  Enable garbage collection after previously disabling it 
	 */
	public void enableGC() {
		if(_isActive && _gcCount>0) {
			_gcCount--;
			enableGC0();
		}
	}
	
	/**
	 * Native delegate of enableGC
	 */
	private native void enableGC0();
	
	public void runGC() {
		if(_isActive) {
			runGC0();
		}
	}
	/**
	 * Native delegate of runGC
	 */
	private native void runGC0();
	
	/**
	 * Is the profiler actively collecting data.
	 */
	public boolean isProfiling() {
		return _isProfiling;
	}
	
	
	/**
	 * Release the profiler so that all of the session data is cleaned up.
	 */
	public void release() {
		if(_isActive) {
			/* Ensure our disable GC count is zero so GC is reenabled */
			while(_gcCount>0) {
				disableGC();
			}
			release0();
		}
	}
	
	/**
	 * Native delegate of release
	 */
	private native void release0();
	
	/** 
	 * emitXMLFragment 
	 * 
	 * @param xmlFragment - the xmlFragment string to emit
	 * @return EMIT_XML_SUCCESS if xmlFragment was successfully written;
	 *         EMIT_XML_SUSPENDED_IO if xmlFragment failed to be written 
	 * 					because monitoring was off
	 *  	   EMIT_XML_FAIL if xmlFragment failed to be written for other reasons      
	 */
	public synchronized static int emitXMLFragment(String xmlFragment) {
		byte[] arr; 
		try {
			arr = xmlFragment.getBytes("UTF-8");
			return emitXMLFragment(arr,0,arr.length);
		}
		catch (java.io.UnsupportedEncodingException e) {
			/* do nothing, since UTF-8 should definitely be supported */
			return EMIT_XML_FAIL;  
		}	 
	}
	
	/** 
	 * emitXMLFragment 
	 * 
	 * @param arr - the byte array where the XML fragment is contained
	 * @param offset - the starting offset into arr where the XML fragment starts
	 * @param len - the length of bytes that should be read from the given offset 
	 * @return EMIT_XML_SUCCESS if xmlFragment was successfully written;
	 *         EMIT_XML_SUSPENDED_IO if xmlFragment failed to be written 
	 * 					because monitoring was off;
	 *  	   EMIT_XML_FAIL if xmlFragment failed to be written for other reasons    
	 */
	public synchronized static int emitXMLFragment(byte[] arr, int offset, int len) {
		return emitXMLFragment0(arr,offset,len); 
	}
	
	/**
	 * Native delegate of emitXMLFragment
	 */
	private native static int emitXMLFragment0(byte[] arr, int offset, int len); 
	
	/**
	 * getCurrentTime
	 * 
	 * Returns current time (in seconds) more accurately than System.currentTimeMillis()
	 */
	public static double getCurrentTime() {
		return getCurrentTime0();
	}
	
	/**
	 * Native delegate of getCurrentTime
	 */
	private native static double getCurrentTime0(); 
	
	/**
	 * getCurrentThreadCpuTime
	 * 
	 * returns thread cpu time converted in seconds (see JVMPI GetCurrentThreadCpuTime).
	 */
	public static double getCurrentThreadCpuTime() {
		return getCurrentThreadCpuTime0();
	}
	
	/**
	 * Native delegate of getCurrentThreadCpuTime
	 */
	private native static double getCurrentThreadCpuTime0(); 
	

}
