package org.eclipse.hyades.logging.adapter.internal.util;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Locale;

import org.eclipse.hyades.logging.core.Guid;
import org.eclipse.hyades.logging.events.cbe.CommonBaseEvent;
import org.eclipse.hyades.logging.events.cbe.CompletionException;
import org.eclipse.hyades.logging.events.cbe.ComponentIdentification;
import org.eclipse.hyades.logging.events.cbe.MsgDataElement;
import org.eclipse.hyades.logging.events.cbe.impl.TemplateContentHandlerImpl;

/*******************************************************************************
 * Copyright (c) 2005 IBM Corporation and others. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: RuntimeTemplateContentHandlerImpl.java,v 1.6 2005/02/16 22:20:29 qiyanli Exp $
 * 
 * Contributors: IBM - Initial API and implementation
 ******************************************************************************/

/**
 * An extension to the <code>TemplateContentHandlerImpl</code> implementation 
 * provided by the GLA to set runtime configuration data on Common Base Events
 * created for logging.
 * <p>
 * 
 * 
 * @author   Paul E. Slauenwhite
 * @version  May 4, 2004
 */
public class RuntimeTemplateContentHandlerImpl extends TemplateContentHandlerImpl {

    /**
     * IP address (IPv4) of the local host, otherwise "127.0.0.1". 
     */
    private static String localHostIP = null;

    static {
        try {
            localHostIP = InetAddress.getLocalHost().getHostAddress();
        }
        catch (UnknownHostException u) {
            localHostIP = "127.0.0.1";
        }
    }

    /**
     * @see org.eclipse.hyades.logging.events.cbe.ContentHandler#completeEvent(org.eclipse.hyades.logging.events.cbe.CommonBaseEvent)
     */
    public void completeEvent(CommonBaseEvent event) throws CompletionException {

        //Only attempt to complete events which exist and have not been previously completed:
        if ((event != null) && (!event.isComplete())) {

            //Call the super class' completeEvent() API to populate the event with pre-runtime configuration data (e.g. predefined configuration template):
            //NOTE:  This call MUST appear first in this over-loaded API to preserve precedence rules for merging event properties.
            super.completeEvent(event);

            //If the parameter event's globalInstanceId property is not set, assign a new GUID:
            if (event.getGlobalInstanceId() == null) {
                event.setGlobalInstanceId(Guid.generate());
            }
            
            //If the parameter event's creationTime property is not set, assign the current time:
            if (!event.isSetCreationTime()) {
                event.setCreationTimeAsLong(System.currentTimeMillis());
            }

            //Attempt to retrieve the event's sourceComponentId:
            ComponentIdentification sourceComponetId = event.getSourceComponentId();

            //If the event's sourceComponentId property is set, update several of its properties:
            if (sourceComponetId != null) {

                //If the event's sourceComponentId threadId property is not set, assign the current thread's name:
                if (sourceComponetId.getThreadId() == null) {
                    sourceComponetId.setThreadId(Thread.currentThread().getName());
                }

                //If the event's sourceComponentId location property is not set, assign the local host's IP address:
                if ((sourceComponetId.getLocation() == null) && (sourceComponetId.getLocationType() == null)) {

                    //Set the event's sourceComponentId location property to the local host's IP address:
                    sourceComponetId.setLocation(localHostIP);

                    //Set the event's sourceComponentId locationType property to 'IPV4':
                    sourceComponetId.setLocationType(ComponentIdentification.LOCATION_TYPE_IPV4);
                }
            }
            
            //Attempt to retrieve the event's msgDataElement:
            MsgDataElement msgDataElement = event.getMsgDataElement();

            //If the event's msgDataElement property is set, update its locale property:
            if (msgDataElement != null) {
                
                //If the event's msgDataElement msgLocale property is not set, assign the current locale:
                if(msgDataElement.getMsgLocale() == null){
                    
                    Locale defaultLocale = Locale.getDefault();
                    
                    msgDataElement.setMsgLocale(defaultLocale.getLanguage().concat("-").concat(defaultLocale.getCountry()));                    
                }
            }            
        }
    }
}
